module Datarobot
  module AiApi
    class Page < Delegator
      attr_reader :total, :items, :klass, :links

      # Creates a paginated collectoin of objects of type klass. Uses a parsed
      # response body of paginated data to initialize group of objects
      def initialize(klass, param_collection)
        @items = param_collection["data"].map do |params|
          klass.new(params)
        end
        @klass = klass
        @total = param_collection["total"]
        if param_collection["links"] # predictions don't have pages
          @links = param_collection["links"]
          @next_page = param_collection["links"]["next"]
          @previous_page = param_collection["links"]["previous"]
        end
      end

      # Goes to the next page in the dataset if there is one. Otherwise,
      # returns the current page
      # @return [Datarobot::AiApi::Page[klass]]
      def next_page
        return self if @next_page.nil?

        Datarobot::AiApi.get(@next_page) do |data|
          self.class.new(@klass, data)
        end
      end

      # Goes to the prevoius page in the dataset if there is one. Otherwise,
      # returns the current page
      # @return [Datarobot::AiApi::Page[klass]]
      def previous_page
        return self if @previous_page.nil?

        Datarobot::AiApi.get(@previous_page) do |data|
          self.class.new(@klass, data)
        end
      end

      # Allows all Array-like methods to be called on this class. They will be
      # delegated to @items
      def __getobj__
        @items
      end
    end
  end
end