module Datarobot
  module AiApi
    class Task
      include Datarobot::AiApi::Refreshable

      attr_reader :id, :code, :status_type, :status_id, :created_at,
      :description, :message, :status, :links

      # Given a parsed response body from the API, will create a new leanring sessoin object
      def initialize(options = {})
        # Suppresses warnings about uninitialized variables
        @code = nil
        @status = nil
        @status_type = nil
        @status_id = nil
        @id = nil
        @created_at = nil
        @description = nil
        @message = nil
        @links = nil

        set_from_options(options)
      end

      # Takes a response body from the API. Will set all task attributes from
      # the response body
      #
      # @param [Hash] options A parsed response body
      # @return [void]
      def set_from_options(options = {})
        # one-liner replacement for `stringify_keys`
        options = options.collect{|k,v| [k.to_s, v]}.to_h

        @code = options.dig("code") || @code
        @status = options.dig("status") || @status
        @status_type = options.dig("statusType") || @status_type
        @status_id = options.dig("statusId") || @status_id
        @id = options.dig("statusId") || @id
        @created_at = options.dig("created") || @created_at
        @description = options.dig("description") || @description
        @message = options.dig("message") || @message
        @links = options.dig("links") || @links
      end

      # Finds a task by id
      #
      # @param [String] id The id of the task to find
      # @return [Datarobot::AiApi::Task]
      def self.find(id, &block)
        raise Datarobot::AiApi::NotFoundError, "Cannot find Task with id: nil" if id.nil?
        Datarobot::AiApi.request_endpoint("/aiapi/status/#{id}") do |data|
          if block_given?
            yield data
          else
            self.new(data)
          end
        end
      end

      # Stops a task by id
      #
      # @param [String] id The id of the task to stop
      # @return [Datarobot::AiApi::Task]
      def self.stop(id)
        Datarobot::AiApi.request_endpoint("/aiapi/status/#{id}", method: "delete")
      end

      # Waits for a task to complete and returns the resulting object.
      #
      # @return [Datarobot::AiApi::*]
      def wait_until_complete
        refreshed = self
        until refreshed.status == 'COMPLETED'
          sleep 1 unless ENV['FAST_TEST_RESPONSE'] == 'true'
          refreshed = Datarobot::AiApi::Task.find(@id)
        end

        result = refreshed.links["result"]
        Datarobot::AiApi.get(result) do |data|
          klass = Datarobot::AiApi.determine_object(result)
          klass.new(data)
        end
      end
    end
  end
end
